%D=mapDisperseFourier(k,z,p)
%---------------------------
%
%Tomogram I(z) from measured spectrum A(k).
%
%Inputs:
% k   Sampled wavevectors [rad/m]
% z   Optical path differences [m]
% p   Dispersion phases [rad]
%
%Output:
% D   Transform matrix D(z,k): I(z) = D(z,k)*A(k)
%
%
%The measured spectrum A(k), where k = 2pi/lambda is the non-uniformly spaced
%wavevector, requires a remaping for an equidistant sampling and a dispersion
%compensation exp(ip(k)), where p(k) is the phase correction. Finally, the
%Fourier transform F(A(k)) yields the tomogram I(z), where z is the optical
%path difference between the sample and the reference arm.
%
% a(k) = A(k)*exp(ip(k))
% I(z) = int{a(k)*exp(-ikz)dk}
%
%We can express a(k) with N sampling points kn, 1 <= n <= N, and N basis
%functions fn(k), such that a(k) ~ sum(an*fn(k)), where an = a(kn). The
%boundary conditions are chosen by a(k <= k0) = a(kN+1 <= k) = 0, where
%k0 = 2*k1 - k2 and kN+1 = 2*kN - kN-1.
%
%          /    0        for           k <= kn-1
%          |
%          | k  - kn-1
%          | ---------   for   kn-1 <= k <= kn
%          | kn - kn-1
% fn(k) = <
%          | kn+1 - k
%          | ---------   for   kn   <= k <= kn+1
%          | kn+1 - kn
%          |
%          \    0        for   kn+1 <= k
%
%Performing the integration
%
%                    i
% int{exp(-ikz)dk} = - exp(-ikz)
%                    z
%
%                      1 + ikz
% int{k*exp(-ikz)dk} = ------- exp(-ikz)
%                        z
%
%yields for the n-th sampling point
%
%                          1  exp(-iknz) - exp(-ikn-1z)   exp(-iknz) - exp(-ikn+1z)
% int{fn(k)*exp(-ikz)dk} = --(------------------------- + -------------------------)
%                          z         kn - kn-1                 kn+1 - kn
%
%Let now define an [M x N] matrix D for the sampling positions zm, where
%1 <= m <= M and
%
% Dmn := exp(ipn)*int{fn(k)*exp(-ikzm)dk}*zm
%
function D=mapDisperseFourier(k,z,p)
if nargin < 3 || isempty(p)
   p=1;
elseif ~isequal(size(k),size(p))
   error('Invalid dimensions of dispersion correction factors.');
else
   p=exp(1i*p(:).');
end
z=z(:);
k=[2*k(1)-k(2);k(:);2*k(end)-k(end-1)].';
D=diff(exp(-1i*z*k),1,2);
k=1./diff(k,1,2);
D=z*(p.*k(1:end-1)).*D(:,1:end-1) - z*(p.*k(2:end)).*D(:,2:end);
