%Es=occsBrightness(rs,ro,wo,zo,lo)
%kz=occsBrightness(Rs,ro,[],[],lo,gpu)
%-------------------------------------
%
%Approximate OCCS brightness profile modelling a Bessel
%illumination profile and a Gaussian detection profile.
%
%  Es(r,z) = Ei(r,z)*Ed(r,z)
%
%  r(x,y) = sqrt(x^2 + y^2)
%
%Bessel illumination profile:
%
%               2.404                          1 - v(z)^2
%  Ei(r,z) = Jo(-----*sqrt(r))*max(0,v(z))*exp(----------)
%                 ro                               2
%
%             z*sqrt(2)
%  v(z) = 1 + ---------
%                zo
%
%Gaussian detection profile:
%
%            w(0)      -r^2
%  Ed(r,z) = ----*exp(------)
%            w(z)     w(z)^2
%
%                     z^2*lo^2
%  w(z) = wo*sqrt(1 + ---------)
%                     pi^2*wo^4
%
%Inputs:
% rs     Coordinate(s) of scatterer [m]
% Rs     Sandbox radius (maximal radius) [m]
% ro     Radius of first Bessel beam zero [m]      {400nm}
% zo     Bessel beam for z in (-0.481,0.652)zo [m] {40um}
%        as defined by beam waist and axicon angle
% wo     Focal waist of the Gaussian detection [m] {1um}
% lo     Central wavelength in sample [m]          {413nm}
% gpu    Force/forbid usage of CUDA capable GPU    {auto}
%        If specified, the radial Bessel profile
%        is cached for the sandbox radius rs.
%
%Output:
% Es     Scattered field
% kz     Axial phase factor <2 due to conical illumination
%
function Es=occsBrightness(rs,ro,wo,zo,lo,gpu)
if nargin < 2 || isempty(ro)
   ro=4e-7;
end
if nargin > 5
   Es=2.404*lo/(2*pi*ro);
   Es=1 + sqrt(1 - Es^2);
   bessel(2.404*rs/ro,gpu);
   return;
end
if nargin < 3 || isempty(wo)
   wo=1e-6;
end
if nargin < 4 || isempty(zo)
   zo=40e-6;
end
if nargin < 5 || isempty(lo)
   lo=550e-9/1.33;
end
s=size(rs);
s(1)=1;
vz=1 + sqrt(2)/zo*rs(3,:);
wz=wo^2*(1 + (lo/(pi*wo^2)*rs(3,:)).^2);
rs=rs(1,:).^2 + rs(2,:).^2;
Es=reshape(wo./sqrt(wz).*bessel(2.404/ro*sqrt(rs)).*max(0,vz).*exp((1-vz.^2)/2-rs./wz),s);


%Approximate radial Bessel profile besselj(0,r).
%
function e=bessel(r,gpu)
persistent Jo
if nargin > 1
   Jo=besselj(0,0:0.002:r+0.01);
   if gpu
      Jo=parallel.gpu.GPUArray(Jo);
   end
elseif isempty(Jo)
   e=besselj(0,r);
else
   e=Jo(1+round(500*r));
end
