%Symetrically normalized quasi-logarithmic correlations.
%  
%Synopsis:
%  [G,tau]=occsCorrelate(traces)
%     Get the normalized auto- and cross-correlations G versus
%     lag times tau. The photon traces are column vectors. The
%     auto-correlations are the diagonal elements of the second
%     and third dimensions of G.
%  
%  [G,tau]=occsCorrelate(traces,points)
%     Specify the number of points per segment (default: 8).
%
%Example:
%  Calculate the auto- and cross-correlations of two signals
%  A(t) and B(t) for 0 < t < T. The correlations G(tau) are
%  returned as [AxA AxB;BxA BxB](tau).
%
%                    sum(A(1:T-tau).*B(tau:T))
%     AxB = (T-tau)-----------------------------
%                  sum(A(1:T-tau))*sum(B(tau:T))
%
%>> traces=[A(:) B(:)];
%>> [G,tau]=occsCorrelate(traces);
%
function [G,tau]=occsCorrelate(traces,points)
if ~ismatrix(traces)
   error('Traces must be a matrix.');
end
if nargin < 2 || isempty(points)
   points=8;
end
if isvector(traces)
   traces=traces(:);
end
if isempty(traces)
   G=zeros(size(traces,2));         % no signals
   tau=0;
   return;
end
time=size(traces,1);                % duration T
tau=pow2(1,0:log2(time/points)-1);
tau=tau(ones(points,1),[1 1:end]);
tau=cumsum([0;tau(:)]);             % lag times
G=size(traces,2);
G=zeros(numel(tau),G,G);            % correlations
%
% Iterate over lag times and sum with matrix product
%
signal=sum(traces,1);               % average in [0..T-tau]
langis=signal';                     % average in [tau..T]
secart=traces';                     % traces in [tau..T]
lag=0;                              % lag time tau
G(1,:,:)=time.*gather((secart*traces)./(langis*signal));
for n=0:numel(tau)-2
   T=size(traces,1);
   if n > points && ~rem(n,points)
      lag=ceil(0.5*lag);            % double sampling step
      time=floor(0.5*time);
      traces=traces(1:2:T-3,:) + traces(2:2:T-2,:);
      secart=secart(:,3:2:T-1) + secart(:,4:2:T);
      signal=sum(traces,1);         % refresh signals
      langis=sum(secart,2);
   else
      signal=signal - traces(T,:);  % rectify signals
      langis=langis - secart(:,1);
      traces=traces(1:T-1,:);       % remove last sample
      secart=secart(:,2:T);         % remove first sample
   end
   lag=lag + 1;
   G(n+2,:,:)=(time-lag).*gather((secart*traces)./(langis*signal));
end
