%[A,z,occs,time]=occsExperiment(time,occs,param,value,...)
%---------------------------------------------------------
%
%Simulate an optical coherence correlation spectroscopy experiment.
%
%Inputs:
% time   Measurement duration [s]
% occs   Simulation parameters
%  ...   See also occsSettings
%
%Outputs:
% A      Amplitude tomogram |A(z)|
% z      Tomogram positions [m]
% occs   Simulation parameters
% time   Simulated measurement time [s]
%
function [A,z,occs,time]=occsExperiment(time,varargin)
fig=statusbar('Initialization...');
occs=occsSettings(varargin{:});
cls=occs.cls;
gpu=occs.gpu;
Np=occs.Np;
Rs=occs.Rs;
Ls=occs.Ls;
dt=occs.dt;
Dp=cast(occs.Dp,cls);
kn=cast(occs.kn,cls);
Ir=cast(occs.Ir,cls);
T=round(time/dt);
Er=sqrt(Ir);
Ir=occsDetect(Ir,occs.dI,occs.In,occs.Io,[],1);
fig=statusbar(1/3,fig);
if gpu
   Er=parallel.gpu.GPUArray(Er);
   Ir=parallel.gpu.GPUArray(Ir);
   kn=parallel.gpu.GPUArray(kn);
%    A=parallel.gpu.GPUArray.zeros(numel(occs.zs),T,cls);
% else
%    A=zeros(numel(occs.zs),T,cls);
end
A=zeros(numel(occs.zs),T,cls);
Ir=Ir/sum(Ir);
%
% Initialize wavevector mapping and Bessel profile.
%
fig=statusbar(2/3,fig);
kz=occsBrightness(Rs,occs.ro,[],[],occs.lo/occs.ns,gpu);
% [~,z]=occsTomogram(Ir,kn,occs.zs);
zs=floor(kz/2*occs.zs(1)):ceil(kz/2*occs.zs(end));
[~,z]=occsTomogram(Ir,kn,zs);
z=2/kz*z;
M=min(max(100,round(1e6/Np)),2000);
%
% Simulate the correlation measurement.
%
m=min(T,M);
rs=zeros(3,m,Np,cls);
fig=statusbar('OCCS simulation...',fig);
for n=1:Np
   rs(:,:,n)=occsTrajectory(Dp,Rs,Ls,m*dt,dt);
end
t=0;
while t < T
   ro=rs(:,end,:);
   if gpu
      rs=parallel.gpu.GPUArray(rs);
   end
   Es=occs.Bp*occsBrightness(rs,occs.ro,occs.wo,occs.zo,occs.lo/occs.ns);
   rs=rs(3,:,:);
   Ic=occsInterferogram(kz,rs,Es,occs.zr,Er,kn,gpu);
   m=min(T-t-m,M);
   if m > 0
      rs=zeros(3,m,Np,cls);
      for n=1:Np
         rs(:,:,n)=occsTrajectory(Dp,Rs,Ls,m*dt,dt,ro(:,n));
      end
   end
   Ic=occsDetect(Ic,occs.dI,occs.In,occs.Io,occs.Nb);
   Ic=Ic - Ir*sum(Ic,1);
   m=size(Ic,2);
%    A(:,1+t:t+m)=occsTomogram(Ic,[],occs.zs);
%    A(:,1+t:t+m)=gather(occsTomogram(Ic,[],occs.zs));
   A(:,1+t:t+m)=gather(occsTomogram(Ic,[],zs));
   t=t + m;
   fig=statusbar(t/T,fig);
   if isempty(fig)
      break;
   end
end
delete(fig);
z=gather(z+occs.zr);
time=t*dt;
