%[Ic,kn,Ir]=occsInterferogram(kz,zs,Es,zr,Er,kn,gpu)
%---------------------------------------------------
%
%Interference spectrum intensity for scattered field(s).
%
%The default source spectrum is a 4th order Gaussian spectrum in the
%range of 450nm to 650nm wavelength with a spectral bandwidth of 116nm
%full-width at half-maximum. The reference field strength is 100 units
%by default.
%
%If the scatterer coordinates and/or fields can be three-dimensional
%matrices for several scatterers together: zs(1,t,s).
%
%The spectrum encodes the interference between the scatterers and the
%reference field and the mutual interference between the scatterers.
%These terms can be separated in the tomograms if the reference mirror
%is put outside the scattering range by more then the maximal distance
%between the scatterers.
%
%Inputs:
% kz     Axial phase factor
% zs     Scatterer position(s) [m]
% Es     Scattered field fraction(s)
% zr     Reference mirror position [m]
% Er     Reference field spectrum            {default}
% kn     Wavevector over spectrum [rad/m]    {default}
% gpu    Use or forbid CUDA capable GPU      {auto}
%
%Outputs:
% Ic     Interference intensity spectrum
% kn     Wavevector over spectrum [rad/m]
% Ir     Reference intensity spectrum
%
function [Ic,kn,Ir]=occsInterferogram(kz,zs,Es,zr,Er,kn,use)
persistent gpu ptx
if nargin > 6
   gpu=use;
end
if isempty(gpu)
   gpu=cudaAvailable;
end
if gpu && isempty(ptx)
   ptx=fileparts(mfilename('fullpath'));
   ptx={parallel.gpu.CUDAKernel(fullfile(ptx,'private','interferogram.ptx'),fullfile(ptx,'private','interferogram.cu'),'interferogram');
        parallel.gpu.CUDAKernel(fullfile(ptx,'private','interferogram.ptx'),fullfile(ptx,'private','interferogram.cu'),'interferogramf')};
   %
   % If linking to the GPU kernel fails, try to recompile interferogram.cu.
   % Change to the private folder and launch the Nvidia compiler. The -arch
   % option should be set to the compute capability of your CUDA-capable
   % graphics card. In the example, it is 2.1:
   %
   % >> !nvcc -arch=sm_21 -ptx interferogram.cu
end
if nargin < 5 || isempty(Er)
   Er=100;
end
if nargin < 6 || isempty(kn)
   kn=652.4:-0.2:447.7;
   if gpu
      kn=gpuArray(kn);
   end
   Er=Er*exp(-2/90.^4*(kn - 550).^4);
   kn=2.66e9*pi./kn;
end
kn=kn(:);
Er=Er(:);
if size(zs,1) == 1
   [~,T,S]=size(zs);
else
   [T,S]=size(zs);
end
if gpu
   M=numel(kn);
   try
      cls=classUnderlying(kn);
   catch
      cls=class(kn);
   end
   Ic=parallel.gpu.GPUArray.zeros(M,T,cls);
   if strcmp(cls,'double')
      fun=ptx{1};
   else
      fun=ptx{2};
   end
   fun.ThreadBlockSize=32*min(4,ceil(M/32));
   fun.GridSize=ceil([M/fun.ThreadBlockSize(1) T]);
   Ic=feval(fun,Ic,Er,kz/pi*kn,Es,zs,[M T S zr]);
else
   zs=reshape(zs,1,numel(zs));
   Es=reshape(Es,size(zs));
   Es=Er*Es.*exp(1i*kz*kn*(zs - zr));
   Es=sum(reshape(Es,numel(kn),T,S),3);
   Ic=abs(Er(:,ones(1,T)) + Es).^2;
end
if nargout > 2
   Ir=abs(Er).^2;
end
