%occs=occsSettings(occs,param,value,...)
%---------------------------------------
%
%Initialize the parameters for an OCCS simulation. The parameters
%are set by named fields in the structure 'occs' or by name/value
%pairs.
%
%Non-specified parameters are looked up in 'occsDefaults.mat'. If
%this attempt fails, defaults are used as given in brackets below.
%
% occs   Simulation parameters
%
%        Brightness profile
%  .ro      Radius of first Bessel beam zero [m]      {400nm}
%  .zo      Bessel beam for z in (-0.481,0.652)zo [m] {40um}
%           as defined by beam waist and axicon angle
%  .wo      Focal waist of the Gaussian detection [m] {1um}
%
%        Detection parameters
%  .Nb      Number of bits                            {12}
%  .Nd      Detector pixels                           {1024}
%  .Io      Detection offset (bias) [unit]            {2*In}
%  .In      Gaussian detector noise [unit]            {1}
%  .dI      Detector sensitivity [unit/photon]        {0.1}
%  .dt      Sampling step/exposure time [s]           {10us}
%  .zr      Reference mirror position [m]             {-130um}
%  .zs      Tomogram position indices                 {sandbox}
%
%        Sample parameters
%  .Bp      Relative brightness of particles          {1%}
%  .Dp      Diffusion constant of particles [m/s]    {10um/s}
%  .Np      Number of particles in sandbox            {100}
%  .cp      Concentration of particles [M]
%  .ns      Refraction index in sample                {1.33}
%
%        Spectrum parameters
%  .dl      FWHM of 4th order Gaussian spectrum [m]   {140nm}
%  .lo      Central wavelength in free space [m]      {550nm}
%  .Ir      Intensity spectrum {peak} [photons]       {10000}
%  .kn      Wavevectors in sample [rad/m]
%
%        General parameters
%  .Rs      Radius of sandbox [m]                     {5um}
%  .Ls      Length of sandbox [m]                     {20um}
%  .cls     Floating point precison (class name)      {single}
%  .gpu     Force/forbid usage of CUDA capable GPU    {auto}
%
function occs=occsSettings(varargin)
occs.gpu=[];
try
   load('occsDefaults.mat','occs');    % load user defaults
catch
end
occs=default(occs,'ro',400e-9);        % brightness defaults
occs=default(occs,'zo',40e-6);
occs=default(occs,'wo',1e-6);
occs=default(occs,'Nb',12);            % detection defaults
occs=default(occs,'Nd',1024);
occs=default(occs,'In',1);
occs=default(occs,'Io',2*occs.In);
occs=default(occs,'dI',0.1);
occs=default(occs,'dt',10e-6);
occs=default(occs,'zr',-130e-6);
occs=default(occs,'zs',[]);
occs=default(occs,'Bp',0.01);          % sample defaults
occs=default(occs,'Dp',10e-12);
occs=default(occs,'Np',100);
occs=default(occs,'ns',1.33);
occs=default(occs,'dl',140e-9);        % spectrum defaults
occs=default(occs,'lo',550e-9);
occs=default(occs,'Ir',10000);
occs=default(occs,'Rs',5e-6);          % general defaults
occs=default(occs,'Ls',20e-6);
occs=default(occs,'cls','single');
occs=default(occs,'gpu',[]);
%
% Apply user settings.
%
n=1;
N=nargin;
while n <= N
   arg=varargin{n};
   if isstruct(arg)
      name=fieldnames(arg);
      for m=1:numel(name)
         occs=setting(occs,name{m},arg.(name{m}));
      end
      n=n + 1;
   elseif ischar(arg) && n < N
      occs=setting(occs,arg,varargin{n+1});
      n=n + 2;
   else
      error('occs:settings','Invalid argument %d: structure or name/value pair expected.',n);
   end
end
if ~any(strcmp(occs.cls,{'double','single'}))
   error('occs:settings','Invalid floating point class: %s.',occs.cls);
end
%
% Match concentration and number of particles.
%
Na=6.0221367e26;           % particles/mmol
Vs=pi*occs.Rs^2*occs.Ls;   % sandbox volume
if isfield(occs,'cp')
   occs.Np=max(1,round(Na*Vs*occs.cp));
end
occs.cp=occs.Np/(Na*Vs);
%
% Determine usage of CUDA capable graphics processor.
%
occs.gpu=(isempty(occs.gpu) || occs.gpu) && cudaAvailable;
%
% Match spectral shape and spectrum parameters.
%
if isfield(occs,'kn') && isequal(size(occs.Ir),size(occs.kn))
   occs.Ir=occs.Ir(:);
   [occs.kn,n]=sort(occs.kn(:));
   occs.Ir=occs.Ir(n);
   lo=2*pi*occs.ns./occs.kn;
   occs.Nd=numel(lo);
   dl=gradient(lo).';
   occs.lo=dl*(lo.*occs.Ir)./(dl*occs.Ir);
else
   if isfield(occs,'kn')
      occs.kn=sort(occs.kn(:));
      dl=2*pi*occs.ns./occs.kn;
      occs.lo=mean(dl([1 end]));
      occs.Nd=numel(dl);
      dl=dl - occs.lo;
   else
      dl=1.5*occs.dl/occs.Nd;
      dl=dl*((1-occs.Nd)/2:occs.Nd/2).';
      occs.kn=2*pi*occs.ns./(occs.lo - dl);
   end
   occs.Ir=occs.Ir*exp(-16*log(2)/occs.dl.^4*dl.^4);
end
%
% Tomogram position indices.
%
if isempty(occs.zs)
   zs=pi/abs(occs.kn(end)-occs.kn(1))*(0:occs.Nd-1);
   occs.zs=find(-occs.Ls/2-occs.zr <= zs & zs <= occs.Ls/2-occs.zr);
end
occs.zs=occs.zs(:);


%Append default parameter if not specified.
%
function occs=default(occs,name,value)
if ~isfield(occs,name)
   occs.(name)=value;
end


%Set a parameter if its name is valid.
%
function occs=setting(occs,name,value)
names=fieldnames(occs);
names(end+1:end+2)={'cp';'kn'};
if any(strcmp(name,names))
   occs.(name)=value;
else
   error('occs:setting','%s is not a valid parameter name.',name);
end
